package ModelInversionControl "Examples of different control setups for controlling the position of a crab crane"
  package Components "Components used in the Model Inversion Control example"
    extends Modelica.Icons.Package;

    model CraneCrabDamped "A model of a simple crane crab where the revolute joint has been removed so that the crab position is always identical to the load position"
      parameter Modelica.Units.SI.Inertia body_I = 0.5 "Interia of the crane body";
      parameter Boolean animateCrane = false "Animation flag";
      PlanarMechanics.Parts.Body loadWithInertia(m = 70, I = body_I, animate = animateCrane) "Load on the crane." annotation(Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {40, -80}), visible = true));
      PlanarMechanics.Parts.FixedTranslation rope(r = {0, -1}, animate = animateCrane) "Stiff rope supporting the load." annotation(Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {40, -40}), visible = true));
      PlanarMechanics.Parts.Fixed wall "Prismatic join fixture." annotation(Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 180, origin = {-40, 40}), visible = true));
      PlanarMechanics.Joints.Prismatic prismatic(r = {1, 0}, useFlange = true, animate = animateCrane, v.fixed = true) "Crab crane movement along X-axis." annotation(Placement(transformation(extent = {{-40, -40}, {-20, -60}}, origin = {30, 90}, rotation = 0), visible = true));
      PlanarMechanics.Parts.Body crab(m = 20, I = 0.5, animate = animateCrane) "Crab moving along the gantry." annotation(Placement(transformation(extent = {{0, 40}, {20, 60}}, origin = {70, -10}, rotation = 0), visible = true));
      Modelica.Blocks.Interfaces.RealOutput loadPosition(unit = "m", start = 0) = loadWithInertia.r[1] "Position of the load" annotation(Placement(visible = true, transformation(origin = {110, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {-100, 0}, extent = {{-10, -10}, {10, 10}}, rotation = -540)), Dialog(group = "Initialization", showStartAttribute = true));
      Modelica.Blocks.Interfaces.RealOutput crabPosition(unit = "m", start = 0) = crab.r[1] "Position of the crab" annotation(Placement(visible = true, transformation(origin = {0, -100}, extent = {{-10, -10}, {10, 10}}, rotation = -90), iconTransformation(origin = {0, -100}, extent = {{-10, -10}, {10, 10}}, rotation = -1170)), Dialog(group = "Initialization", showStartAttribute = true));
      Modelica.Mechanics.Translational.Sources.Force force "Force applied on the crab crane." annotation(Placement(visible = true, transformation(origin = {-40, 80}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Blocks.Interfaces.RealInput inputForce(unit = "N") "Force signal (in newtons) to be applied on the crab" annotation(Placement(visible = true, transformation(origin = {-107.084, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 0), iconTransformation(origin = {99.821, -0.179}, extent = {{-10.179, -10.179}, {10.179, 10.179}}, rotation = -540)));
      PlanarMechanics.Joints.Revolute revolute(useFlange = true, animate = animateCrane, w.fixed = true) "Crab-crane relative angle." annotation(Placement(visible = true, transformation(origin = {40, 0}, extent = {{10, -10}, {-10, 10}}, rotation = 90)));
      Modelica.Mechanics.Rotational.Components.Fixed fixed "Damper fixture." annotation(Placement(visible = true, transformation(origin = {80, -70}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Damper damper(d = 10) "Damper to remove oscillations." annotation(Placement(visible = true, transformation(origin = {80, -26.959}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
      Modelica.Units.SI.Angle loadAngle(start = 0) = revolute.phi "Angle of the load relative to the crab" annotation(Dialog(group = "Initialization", showStartAttribute = true));
    equation
      connect(damper.flange_b, fixed.flange) annotation(Line(visible = true, origin = {80, -53.48}, points = {{0, 16.52}, {0, -16.52}}, color = {64, 64, 64}));
      connect(revolute.flange_a, damper.flange_a) annotation(Line(visible = true, origin = {63.2, 10}, points = {{-13.2, -10}, {16.8, -10}, {16.8, -26.959}}, color = {64, 64, 64}));
      connect(revolute.frame_a, prismatic.frame_b) annotation(Line(visible = true, origin = {30, 30}, points = {{10, -20}, {10, 10}, {-20, 10}}, color = {95, 95, 95}));
      connect(rope.frame_a, revolute.frame_b) annotation(Line(visible = true, origin = {40, -20}, points = {{0, -10}, {0, 10}}, color = {95, 95, 95}));
      connect(force.f, inputForce) annotation(Line(visible = true, origin = {-55.72, 40}, points = {{3.72, 40}, {-1.033, 40}, {-1.033, -40}, {-51.364, -40}}, color = {0, 36, 164}));
      connect(force.flange, prismatic.flange_a) annotation(Line(visible = true, origin = {-10, 70}, points = {{-20, 10}, {10, 10}, {10, -20}}, color = {0, 127, 0}));
      connect(rope.frame_b, loadWithInertia.frame_a) annotation(Line(points = {{0, 10}, {0, -10}}, color = {95, 95, 95}, thickness = 0.5, visible = true, origin = {40, -60}));
      connect(crab.frame_a, prismatic.frame_b) annotation(Line(points = {{30, 0}, {-30, 0}}, color = {95, 95, 95}, thickness = 0.5, visible = true, origin = {40, 40}));
      connect(wall.frame, prismatic.frame_a) annotation(Line(visible = true, origin = {-20, 40}, points = {{-10, 0}, {10, 0}}, color = {95, 95, 95}));
      annotation(Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, fillColor = {171, 203, 255}, fillPattern = FillPattern.Solid, lineThickness = 1.5, extent = {{-100, -100}, {100, 100}}), Text(visible = true, origin = {-44.045, 0}, rotation = -360, extent = {{-34.045, -29.591}, {34.045, 29.591}}, textString = "loadPos", fontSize = 48, textStyle = {TextStyle.Bold}), Text(visible = true, origin = {-0, -69.056}, rotation = -360, extent = {{-33.011, -20.944}, {33.011, 20.944}}, textString = "crabPos", fontSize = 48, textStyle = {TextStyle.Bold}), Text(visible = true, origin = {60, -0.944}, rotation = -360, extent = {{-20, -20.944}, {20, 20.944}}, textString = "force", fontSize = 48, textStyle = {TextStyle.Bold}), Line(visible = true, origin = {-80.992, 54.2}, points = {{0, 19.226}, {0, -19.226}}), Rectangle(visible = true, origin = {34.008, 53.638}, fillColor = {217, 217, 217}, fillPattern = FillPattern.Solid, extent = {{-45, -15.961}, {45, 15.961}}), Line(visible = true, origin = {19.285, 17.5}, points = {{17.291, 22.099}, {-17.291, -22.099}}, thickness = 2), Ellipse(visible = true, origin = {0, -8.853}, fillColor = {217, 217, 217}, fillPattern = FillPattern.Sphere, extent = {{-6.052, -5.745}, {6.052, 5.745}}), Ellipse(visible = true, origin = {36.423, 39.599}, fillColor = {217, 217, 217}, fillPattern = FillPattern.Sphere, extent = {{-7.415, -6.742}, {7.415, 6.742}}), Rectangle(visible = true, origin = {-65.992, 54.599}, fillColor = {217, 217, 217}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-15, -5}, {15, 5}}), Rectangle(visible = true, origin = {-30.992, 54.509}, fillColor = {217, 217, 217}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-20, -2.111}, {20, 2.111}}), Line(visible = true, origin = {-85.496, 69.598}, points = {{4.504, 5}, {-4.504, -5}}), Line(visible = true, origin = {-85.496, 54.599}, points = {{4.504, 5}, {-4.504, -5}}), Line(visible = true, origin = {-85.496, 39.599}, points = {{4.504, 5}, {-4.504, -5}})}), Diagram(graphics), __Dymola_experimentSetupOutput, Documentation(info = "<html>

    <p>
The hoist crane is modeled using the PlanarMechanics Modelica library. Force is applied to a prismatic joint that pushes a crab along a one-dimensional beam. Attached to the crab is a dampened revolute joint that connects to the load through a stiff rod.
</p></html>"));
    end CraneCrabDamped;

    partial model PartialControlledCrane "Partial model for the controlled crane setup"
      Modelica.Blocks.Continuous.Filter filter(f_cut = inputFilter, order = 4) "Input signal filter, smoothing the signal." annotation(Placement(visible = true, transformation(origin = {-120, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 0)));
      Modelica.Blocks.Sources.Pulse reference(amplitude = 1, period = pulsePeriod, startTime = 2) "Load position reference, going back and forth between 0 and 1 meters along the X-axis." annotation(Placement(visible = true, transformation(origin = {-200, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 0)));
      inner PlanarMechanics.PlanarWorld planarWorld(enableAnimation = true) "Coordinate system." annotation(Placement(visible = true, transformation(origin = {-200, 60}, extent = {{-20, -20}, {20, 20}}, rotation = 0)));
      CraneCrabDamped crane(animateCrane = true, body_I = crane_I) "Crab crane setup." annotation(Placement(visible = true, transformation(origin = {200, 0}, extent = {{20, -20}, {-20, 20}}, rotation = 0)));
      parameter Modelica.Units.SI.Time pulsePeriod = 20 "Period of the input pulse, position switches every half period" annotation(Dialog(group = "Input Properties"));
      parameter Modelica.Units.SI.Frequency inputFilter = 0.2 "Cut-off frequency of the input filter, higher values give sharper movement" annotation(Dialog(group = "Input Properties"));
      parameter Modelica.Units.SI.Inertia crane_I = 0.5 "Inertia of the physical crane body" annotation(Dialog(group = "Crane Properties"));
      Modelica.Units.SI.Position filterOutput = filter.y "Connector of Real output signal (filter.y)";
      Modelica.Units.SI.Position referenceOutput = reference.y "Connector of Real output signal (reference.y)";
      Modelica.Units.SI.Position loadPosition(start = 0) = crane.loadPosition "Lateral position of the load" annotation(Dialog(group = "Initialization", showStartAttribute = true));
      Modelica.Units.SI.Position crabPosition(start = 0, fixed = true) = crane.crabPosition "Lateral position of the crab" annotation(Dialog(group = "Initialization", showStartAttribute = true));
      Modelica.Units.SI.Force inputForce = crane.inputForce "Force signal applied on the crab (crane.inputForce)";
      Modelica.Units.SI.Velocity v = crane.prismatic.v "Velocity of the crab (crane.prismatic.v)";
      Modelica.Units.SI.Acceleration a = crane.prismatic.a "Acceleration of the crab (crane.prismatic.a)";
      Modelica.Units.SI.Angle loadAngle(start = 0, fixed = true) = crane.revolute.phi "Angular position of the load relative to the crab" annotation(Dialog(group = "Initialization", showStartAttribute = true));
      Modelica.Units.SI.AngularVelocity w = crane.revolute.w "Angular velocity of the load (crane.revolute.w)";
      Modelica.Units.SI.AngularAcceleration z = crane.revolute.z "Angular acceleration of the load (crane.revolute.z)";
      Modelica.Units.SI.Position craneCoordinates[2] = crane.loadWithInertia.r "transl. position (crane.loadWithInertia.r)";
      Modelica.Units.SI.Position crabCoordinates[2] = crane.crab.r "transl. position (crane.crab.r)";
    equation
      connect(reference.y, filter.u) annotation(Line(visible = true, origin = {-161, 0}, points = {{-17, 0}, {17, 0}}, color = {0, 0, 127}));
      annotation(Documentation(info = "<html>

    <p>
Partial model for the base setups used in the <a href=\"modelica://ModelInversionControl.InversionControlledCrane\">InversionControlledCrane</a> and <a href=\"modelica://ModelInversionControl.PIDControlledCrane\">PIDControlledCrane</a> examples. Contains the crane crab setup and filtered reference signal.
</p></html>", figures = {Figure(title = "Actual and reference crane position", identifier = "f0815", preferred = true, plots = {Plot(curves = {Curve(y = filterOutput, legend = "Reference Position"), Curve(y = crabPosition, legend = "Crane Position")})}), Figure(title = "Crab and crane X-Y position", identifier = "777e5", plots = {Plot(curves = {Curve(x = crabCoordinates[1], y = crabCoordinates[2], legend = "X-Y Position of Crab"), Curve(x = craneCoordinates[1], y = craneCoordinates[2], legend = "X-Y Position of Crane")})}), Figure(title = "Crab and crane coordinates", identifier = "bfd62", plots = {Plot(curves = {Curve(y = crabCoordinates[1], legend = "Crab X-position"), Curve(y = craneCoordinates[1], legend = "Crane X-position")}), Plot(curves = {Curve(y = crabCoordinates[2], legend = "Crab Y-position"), Curve(y = craneCoordinates[2], legend = "Crane Y-position")})}), Figure(title = "Reference signal filter", identifier = "33f29", plots = {Plot(curves = {Curve(y = referenceOutput, legend = "Raw Reference Signal"), Curve(y = filterOutput, legend = "Filtered Reference Signal")})}), Figure(title = "Crab force", identifier = "ca243", plots = {Plot(curves = {Curve(y = inputForce, legend = "Force on Crab")})}), Figure(title = "Crane states", identifier = "a9207", plots = {Plot(curves = {Curve(y = loadAngle, legend = "Crane Angle"), Curve(y = w, legend = "Crane Angular Velocity"), Curve(y = z, legend = "Crane Angular Acceleration")})}), Figure(title = "Crab states", identifier = "8f64e", plots = {Plot(curves = {Curve(y = crabPosition, legend = "Crab Position"), Curve(y = v, legend = "Crab Velocity"), Curve(y = a, legend = "Crab Acceleration")})})}), __Wolfram, Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Ellipse(visible = true, lineColor = {75, 138, 73}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-100, -100}, {100, 100}}), Polygon(visible = true, lineColor = {0, 0, 255}, fillColor = {75, 138, 73}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-36, 60}, {64, 0}, {-36, -60}, {-36, 60}})}), Diagram(coordinateSystem(extent = {{-240, -105}, {240, 105}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
    end PartialControlledCrane;
    annotation(Documentation, Diagram(coordinateSystem(extent = {{-148.5, -105}, {148.5, 105}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
  end Components;

  model InversionControlledCrane "Crab crane with a model inversion control setup"
    extends Components.PartialControlledCrane(loadAngle.fixed = true, crabPosition.fixed = false, loadPosition.fixed = false);
    Modelica.Blocks.Math.Feedback feedback "Corrective closed loop feedback of crane position." annotation(Placement(visible = true, transformation(origin = {200, -60}, extent = {{20, 20}, {-20, -20}}, rotation = -720)));
    Modelica.Blocks.Math.InverseBlockConstraints invertedModel "Crab crane inversion block. Open loop control." annotation(Placement(visible = true, transformation(origin = {-0, 0}, extent = {{-60, -40}, {60, 40}}, rotation = 0)));
    Modelica.Blocks.Continuous.PID feedbackController(k = k_PID, Ti = Ti_PID, Td = Td_PID, I.k = if useCorrectiveController then feedbackController.unitTime / feedbackController.Ti else 0, P.k = if useCorrectiveController then feedbackController.k else 0, D.k = if useCorrectiveController then feedbackController.Td / feedbackController.unitTime else 0, initType = initType, y.start = 0, y.fixed = true) "Corrective PID-controller." annotation(Placement(visible = true, transformation(origin = {120, -60}, extent = {{-20, 20}, {20, -20}}, rotation = -180)));
    Modelica.Blocks.Math.Add add(k1 = +1, k2 = if useCorrectiveController then 1 else 0) "Input force sum." annotation(Placement(visible = true, transformation(origin = {120, -14.572}, extent = {{-20, -5.428}, {20, 34.572}}, rotation = 0)));
    ModelInversionControl.Components.CraneCrabDamped craneControlModel(body_I = craneModel_I) "Crab crane model." annotation(Placement(visible = true, transformation(origin = {-0, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 0)));
    parameter Modelica.Units.SI.Inertia craneModel_I = 0.5 "Inertia of the inverted crane body model." annotation(Dialog(group = "Controller Properties"));
    parameter Boolean useCorrectiveController = false "Complement open loop control with closed loop corrective controller." annotation(Dialog(group = "Controller Properties"));
    parameter Real k_PID = 100 "Proportional gain of corrective controller." annotation(Dialog(group = "Controller Properties"));
    parameter Real Ti_PID = 100 "Time constant of integrative part of corrective controller." annotation(Dialog(group = "Controller Properties"));
    parameter Real Td_PID = 0.1 "Time constant of derivative part of corrective controller." annotation(Dialog(group = "Controller Properties"));
    parameter Modelica.Blocks.Types.Init initType = Modelica.Blocks.Types.Init.SteadyState "Type of initialization used for PID controller" annotation(Dialog(group = "Initialization"));
  equation
    connect(craneControlModel.crabPosition, feedback.u1) annotation(Line(visible = true, origin = {140.57, -147.438}, points = {{-140.57, 127.438}, {-140.57, 57.438}, {89.43, 57.438}, {89.43, 87.438}, {75.43, 87.438}}, color = {1, 37, 163}));
    connect(craneControlModel.inputForce, invertedModel.y2) annotation(Line(visible = true, origin = {30.309, -0.012}, points = {{-10.345, -0.024}, {-10.345, 0.012}, {20.691, 0.012}}, color = {0, 36, 164}));
    connect(invertedModel.u2, craneControlModel.loadPosition) annotation(Line(visible = true, origin = {-38.667, 0}, points = {{-9.333, 0}, {-9.333, -0}, {18.667, -0}}, color = {0, 36, 164}));
    connect(feedbackController.y, add.u2) annotation(Line(visible = true, origin = {82, -36}, points = {{16, -24}, {-2, -24}, {-2, 24}, {14, 24}}, color = {1, 37, 163}));
    connect(feedback.y, feedbackController.u) annotation(Line(visible = true, origin = {163, -60}, points = {{19, 0}, {-19, 0}}, color = {1, 37, 163}));
    connect(invertedModel.y1, add.u1) annotation(Line(visible = true, origin = {71.762, 3}, points = {{-8.762, -3}, {8.238, -3}, {8.238, 9}, {24.238, 9}}, color = {1, 37, 163}));
    connect(filter.y, invertedModel.u1) annotation(Line(visible = true, origin = {-82, 0}, points = {{-16, 0}, {16, 0}}, color = {0, 0, 127}));
    connect(add.y, crane.inputForce) annotation(Line(visible = true, origin = {168.934, -0.018}, points = {{-26.934, 0.018}, {7.916, 0.018}, {7.916, -0.018}, {11.102, -0.018}}, color = {0, 0, 127}));
    connect(crane.crabPosition, feedback.u2) annotation(Line(visible = true, origin = {200, -32}, points = {{0, 12}, {0, -12}}, color = {0, 0, 127}));
    annotation(preferredView = "info", experiment(StopTime = 80, __Wolfram_SynchronizeWithRealTime = false), __Wolfram, Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})), Diagram(coordinateSystem(extent = {{-240, -100}, {240, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})), __Dymola_experimentSetupOutput, Documentation(info = "<html>
<div style=\"background-color: #f4fbff; padding: 1rem; border: 2px solid #d9e5f3\">

<h2>Library Dependency</h2>
    <p class=\"\">
		This model requires the <a href=\"https://www.wolfram.com/system-modeler/libraries/planar-mechanics/\">PlanarMechanics</a> library.
	</p>
	<ul>
		<li style=\"background: url('modelica://EducationExamples/Resources/PlanarMechanics.png') no-repeat left top; background-size: 32px 32px; padding-left: 40px; \">The free PlanarMechanics library was created especially for modeling multibody systems with two-dimensional mechanical components. Compared to the MultiBody library, currently available in the Modelica Standard Library, it is simpler to use and it is more optimized to planar modeling. Planar models of mechanical systems are useful in many different applications, for example, in contact problems that are more easily modeled in 2D than in 3D.</li>

	</ul>
</div>
<h4>Model</h4>
    <p>
This model shows an inversion-based controller that is used to control a hoist crane.
</p>



    <h4>Dynamics</h4>
    <p>
The <a href=\"modelica://ModelInversionControl.Components.CraneCrabDamped\">hoist crane</a> is modeled using the PlanarMechanics Modelica library. Force is applied to a prismatic joint that pushes a crab along a one-dimensional beam. Attached to the crab is a dampened revolute joint that connects to the load through a stiff rod.
</p><p>
In the controller, a filtered pulse is used as a reference signal to control the position of the load. Using a <a href=\"modelica://Modelica.Blocks.Math.InverseBlockConstraints\">constrained inverse block</a>, the force that should be applied to the crab can be calculated from the reference signal.
</p>
    <h4>Simulation</h4>
    <p>
To simulate the model, click the Simulate button:<img src=\"modelica://ModelInversionControl/DocumentationFiles/play.svg\" style=\"width:20px\" alt=\"simulate\"/>


<h4>Plot the Results</h4>
    <p>
Explore how the crane is positioned relative to its reference position by examining the plot that is automatically displayed after simulating the model. The first variable describes the actual position of the crane, while the second describes the reference position that is input into the controller.
</p><p>

<p style=\"text-align: center;\"><img src=\"modelica://ModelInversionControl/DocumentationFiles/modelinversiondefaultplot.png\" style=\"width:600px\" alt=\"\" /></p>
</p><h4>Automatic 3D Animation</h4>
    <p>
PlanarMechanics and Multibody systems have visualizers to show what a real-world system would look like.
</p>
<p style=\"text-align: center;\"><img src=\"modelica://ModelInversionControl/DocumentationFiles/sim.png\" style=\"width:400px\" alt=\"\" /></p>
<p>
To show a 3D animation of the model, click the Animation button: <img src=\"modelica://ModelInversionControl/DocumentationFiles/animate.svg\" style=\"width:20px\" alt=\"animate\"/> </p>
<p>Use your mouse or trackpad to drag the animation to a good angle and zoom in with your scroll wheel or by using the trackpad.</p>
<h4>Change Parameters</h4>
    <p>
Changing parameters for the simulation can be done rapidly in Simulation Center. The <span>crane_I</span> parameter determines the \"real\" inertia of the load. The model used inside the controller also has an inertia parameter that determines what the controller expects the load inertia to be. When the real and the expected inertia differ from each other, the controller will not be able to accurately predict what force to apply to the crane.
</p><p>
Try changing the inertia for the load and see how this affects the controller:
</p><ul><li>Switch to the <span>Parameter</span> tab on the left in the Experiment Browser.</li><li>Cange the value next to the <span>crane_I</span> parameter to 1000 kg.m2.</li><li>Simulate again by following the steps in the Simulation section to see the effects of your changes.</li></ul><p>
You should now see the graph below:
</p>
<p style=\"text-align: center;\"><img src=\"modelica://ModelInversionControl/DocumentationFiles/defaultplotwithchangedparameters.png\" style=\"width:600px\" alt=\"\" /></p>



<p>
In order to get the full experience of this example, you need a desktop Wolfram Language product. A free trial download is available at <a href=\"https://www.wolfram.com/mathematica/trial/\">www.wolfram.com/mathematica/trial/</a>
</p>
<p>
For the full example, open the accompanying notebook <a href=\"ModelInversionControl.nb\">ModelInversionControl.nb</a>.
</p>

</html>", figures = {Figure(title = "PID controller states", identifier = "1b743", plots = {Plot(curves = {Curve(y = feedbackController.P.y, legend = "P"), Curve(y = feedbackController.I.y, legend = "I"), Curve(y = feedbackController.D.y, legend = "D")}, y = Axis(label = "[N]")), Plot(curves = {Curve(y = feedbackController.y, legend = "Output")}, y = Axis(label = "Corrective Force [N]")), Plot(curves = {Curve(y = feedbackController.u, legend = "Input")}, y = Axis(label = "Position Difference from Reference [m]"))}), Figure(title = "Force components", identifier = "106e2", plots = {Plot(curves = {Curve(y = add.u1, legend = "Inverted Model Force Component"), Curve(y = add.u2, legend = "PID Force Component"), Curve(y = add.y, legend = "Total Force")})})}));
  end InversionControlledCrane;

  model PIDControlledCrane "Crab crane with PID-controller"
    extends Components.PartialControlledCrane(crabPosition.fixed = false, loadPosition.fixed = false);
    extends Modelica.Icons.Example;
    parameter Real k_PID = 292.758 "Proportional gain of corrective controller" annotation(Dialog(group = "Controller Properties"));
    parameter Real Ti_PID = 2.06238 "Time constant of integrative part of corrective controller" annotation(Dialog(group = "Controller Properties"));
    parameter Real Td_PID = 0.919468 "Time constant of derivative part of corrective controller" annotation(Dialog(group = "Controller Properties"));
    Modelica.Blocks.Math.Feedback feedback "Corrective closed loop feedback of crane position" annotation(Placement(visible = true, transformation(origin = {0, 0}, extent = {{-20, -20}, {20, 20}}, rotation = -720)));
    Modelica.Blocks.Continuous.PID feedbackController(k = k_PID, Ti = Ti_PID, Td = Td_PID, initType = initType, y.start = 0, y.fixed = true) "Corrective PID-controller" annotation(Placement(visible = true, transformation(origin = {120, 0}, extent = {{20, 20}, {-20, -20}}, rotation = 180)));
    parameter Modelica.Blocks.Types.Init initType = Modelica.Blocks.Types.Init.SteadyState "Type of initialization used for PID controller" annotation(Dialog(group = "Initialization"));
  equation
    connect(feedback.y, feedbackController.u) annotation(Line(visible = true, origin = {57, 0}, points = {{-39, 0}, {39, 0}}, color = {1, 37, 163}));
    connect(filter.y, feedback.u1) annotation(Line(visible = true, origin = {-57, 0}, points = {{-41, 0}, {41, 0}}, color = {0, 0, 127}));
    connect(crane.crabPosition, feedback.u2) annotation(Line(visible = true, origin = {100, -49}, points = {{100, 29}, {100, -31}, {-100, -31}, {-100, 33}}, color = {0, 0, 127}));
    connect(feedbackController.y, crane.inputForce) annotation(Line(visible = true, origin = {168.934, -0.018}, points = {{-26.934, 0.018}, {7.916, 0.018}, {7.916, -0.018}, {11.102, -0.018}}, color = {0, 0, 127}));
    annotation(preferredView = "info", __Wolfram, experiment(StopTime = 80, __Wolfram_SynchronizeWithRealTime = false), Documentation(info = "<html>
<div style=\"background-color: #f4fbff; padding: 1rem; border: 2px solid #d9e5f3\">

<h2>Library Dependency</h2>
    <p class=\"\">
		This model requires the <a href=\"https://www.wolfram.com/system-modeler/libraries/planar-mechanics/\">PlanarMechanics</a> library.
	</p>
	<ul>
		<li style=\"background: url('modelica://EducationExamples/Resources/PlanarMechanics.png') no-repeat left top; background-size: 32px 32px; padding-left: 40px; \">The free PlanarMechanics library was created especially for modeling multibody systems with two-dimensional mechanical components. Compared to the MultiBody library, currently available in the Modelica Standard Library, it is simpler to use and it is more optimized to planar modeling. Planar models of mechanical systems are useful in many different applications, for example, in contact problems that are more easily modeled in 2D than in 3D.</li>

	</ul>
</div>
<h4>Model</h4>
    <p>
Alternative control scheme from the <a href=\"modelica://ModelInversionControl.InversionControlledCrane\">InversionControlledCrane</a> model. Uses a <a href=\"modelica://Modelica.Blocks.Continuous.PID\">PID Controller</a> that has been tuned using the accompanying Wolfram Language notebook.
</p>


<p>
In order to get the full experience of this example, you need a desktop Wolfram Language product. A free trial download is available at <a href=\"https://www.wolfram.com/mathematica/trial/\">www.wolfram.com/mathematica/trial/</a>
</p>
<p>
For the full example, open the accompanying notebook <a href=\"ModelInversionControl.nb\">ModelInversionControl.nb</a>.
</p>






</html>", figures = {Figure(title = "PID controller states", identifier = "1f2ee", plots = {Plot(curves = {Curve(y = feedbackController.P.y, legend = "P"), Curve(y = feedbackController.I.y, legend = "I"), Curve(y = feedbackController.D.y, legend = "D")}, y = Axis(label = "[N]")), Plot(curves = {Curve(y = feedbackController.y, legend = "Output")}, y = Axis(label = "Corrective Force [N]")), Plot(curves = {Curve(y = feedbackController.u, legend = "Input")}, y = Axis(label = "Position Difference from Reference [m]"))})}), Diagram(coordinateSystem(extent = {{-240, -100}, {240, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})));
  end PIDControlledCrane;
  annotation(uses(Modelica(version = "4.0.0"), PlanarMechanics(version = "1.6.0")), preferredView = "info", Documentation(info = "<p>
In order to get the full experience of this example, you need a desktop Wolfram Language product. A free trial download is available at <a href=\"https://www.wolfram.com/mathematica/trial/\">www.wolfram.com/mathematica/trial/</a>
</p>
<p>
For the full example, open the accompanying notebook <a href=\"ModelInversionControl.nb\">ModelInversionControl.nb</a>.
</p>

<div style=\"background-color: #f4fbff; padding: 1rem; border: 2px solid #d9e5f3\">

<h2>Library Dependency</h2>
    <p class=\"\">
		This model requires the <a href=\"https://www.wolfram.com/system-modeler/libraries/planar-mechanics/\">PlanarMechanics</a> library.
	</p>
	<ul>
		<li style=\"background: url('modelica://EducationExamples/Resources/PlanarMechanics.png') no-repeat left top; background-size: 32px 32px; padding-left: 40px; \">The free PlanarMechanics library was created especially for modeling multibody systems with two-dimensional mechanical components. Compared to the MultiBody library, currently available in the Modelica Standard Library, it is simpler to use and it is more optimized to planar modeling. Planar models of mechanical systems are useful in many different applications, for example, in contact problems that are more easily modeled in 2D than in 3D.</li>

	</ul>
</div>"), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, lineColor = {111, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, lineThickness = 4, extent = {{-90, -90}, {90, 90}}, radius = 25), Text(visible = true, origin = {12, 20.833}, textColor = {255, 255, 255}, extent = {{-102, -70.833}, {78, 29.167}}, textString = "Ex", fontName = "Arial")}), __Wolfram(openClass = ModelInversionControl.InversionControlledCrane), Diagram(coordinateSystem(extent = {{-148.5, -105}, {148.5, 105}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
end ModelInversionControl;
